import * as request from 'supertest';
import { createServer } from '@marblejs/core';
import { createHttpServerTestBed } from '@marblejs/core/dist/+internal/testing';
import { verifyToken } from '../src';
import { listener, SECRET_KEY } from './jwt.integration';

const LOGIN_CREDENTIALS = { email: 'admin@admin.com', password: 'admin' };

describe('@marblejs/middleware-jwt - HTTP integration', () => {
  const server = createServer({ listener });
  const httpTestBed = createHttpServerTestBed(server);

  test('POST /api/login returns token and verifies its correctness', async () =>
    request(httpTestBed.getInstance())
      .post('/api/login')
      .send(LOGIN_CREDENTIALS)
      .expect(200)
      .then(req => {
        const { token } = req.body;
        expect(token).toBeDefined();
        expect(typeof token).toBe('string');

        const payload = verifyToken<any>({ secret: SECRET_KEY })(token);
        expect(payload.id).toEqual('test_id');
        expect(payload.email).toEqual('admin@admin.com');
      }));

    test('GET /api/secured authorizes request and checks the `req.user` object', async () => {
      const token = await request(httpTestBed.getInstance())
        .post('/api/login')
        .send(LOGIN_CREDENTIALS)
        .expect(200)
        .then(req => req.body.token as string);

      return request(httpTestBed.getInstance())
        .get('/api/secured')
        .set('Authorization', `Bearer ${token}`)
        .expect(200)
        .then(req => {
          const user = req.body;
          expect(user.id).toEqual('test_id');
          expect(user.email).toEqual(LOGIN_CREDENTIALS.email);
          expect(user.password).toBeUndefined();
        });
    });

    test('GET /api/secured return 401 if payload doesn\'t pass verification', async () => {
      const token = await request(httpTestBed.getInstance())
        .post('/api/login')
        .send({ ...LOGIN_CREDENTIALS, email: 'doesnt_exists@admin.com' })
        .expect(200)
        .then(req => req.body.token as string);

      return request(httpTestBed.getInstance())
        .get('/api/secured')
        .set('Authorization', `Bearer ${token}`)
        .expect(401, { error: { status: 401, message: 'Unauthorized' } });
    });

    test('GET /api/secured returns 401 if token is wrong', async () =>
      request(httpTestBed.getInstance())
        .get('/api/secured')
        .set('Authorization', `Bearer WRONG_TOKEN`)
        .expect(401, { error: { status: 401, message: 'Unauthorized' } }));
});
